@@ -11,8 +11,10 @@ import traceback
 
 from mautrix.util import markdown
 from mautrix.types import RoomID, EventID, MessageEventContent
-from mautrix.appservice import AppService
+from mautrix.appservice import AppService, IntentAPI
+
 from ..user import BaseUser
+from ..portal import BasePortal
 from ..config import BaseBridgeConfig
 
 if TYPE_CHECKING:
@@ -22,9 +24,11 @@ command_handlers: Dict[str, 'CommandHandler'] = {}
 command_aliases: Dict[str, 'CommandHandler'] = {}
 
 HelpSection = NamedTuple('HelpSection', name=str, order=int, description=str)
-HelpCacheKey = NamedTuple('HelpCacheKey', is_management=bool, is_portal=bool)
+HelpCacheKey = NamedTuple('HelpCacheKey', is_management=bool, is_portal=bool,
+                          is_admin=bool, is_logged_in=bool)
 
 SECTION_GENERAL = HelpSection("General", 0, "")
+SECTION_ADMIN = HelpSection("Administration", 50, "")
 
 
 def ensure_trailing_newline(s: str) -> str:
@@ -45,12 +49,13 @@ class CommandEvent:
         command: The issued command.
         args: Arguments given with the issued command.
         content: The raw content in the command event.
-        is_management: Determines whether the room in which the command wa
-            issued is a management room.
-        is_portal: Determines whether the room in which the command was issued
-            is a portal.
+        portal: The portal the command was sent to.
+        is_management: Determines whether the room in which the command was
+            issued in is a management room.
+        has_bridge_bot: Whether or not the bridge bot is in the room.
     """
 
+    bridge: 'Bridge'
     az: AppService
     log: logging.Logger
     loop: asyncio.AbstractEventLoop
@@ -63,12 +68,14 @@ class CommandEvent:
     command: str
     args: List[str]
     content: MessageEventContent
+    portal: Optional[BasePortal]
     is_management: bool
-    is_portal: bool
+    has_bridge_bot: bool
 
     def __init__(self, processor: 'CommandProcessor', room_id: RoomID, event_id: EventID,
                  sender: 'BaseUser', command: str, args: List[str], content: MessageEventContent,
-                 is_management: bool, is_portal: bool) -> None:
+                 portal: Optional[BasePortal], is_management: bool, has_bridge_bot: bool) -> None:
+        self.bridge = processor.bridge
         self.az = processor.az
         self.log = processor.log
         self.loop = processor.loop
@@ -81,8 +88,13 @@ class CommandEvent:
         self.command = command
         self.args = args
         self.content = content
+        self.portal = portal
         self.is_management = is_management
-        self.is_portal = is_portal
+        self.has_bridge_bot = has_bridge_bot
+
+    @property
+    def is_portal(self) -> bool:
+        return self.portal is not None
 
     async def get_help_key(self) -> HelpCacheKey:
         """
@@ -99,7 +111,9 @@ class CommandEvent:
         When you override this property or otherwise extend CommandEvent, remember to pass the
         extended CommandEvent class when initializing your CommandProcessor.
         """
-        return HelpCacheKey(is_management=self.is_management, is_portal=self.is_portal)
+        return HelpCacheKey(is_management=self.is_management, is_portal=self.portal is not None,
+                            is_admin=self.sender.is_admin,
+                            is_logged_in=await self.sender.is_logged_in())
 
     @property
     def print_error_traceback(self) -> bool:
@@ -110,7 +124,12 @@ class CommandEvent:
 
         Bridges may want to limit tracebacks to bridge admins.
         """
-        return self.is_management
+        return self.sender.is_admin
+
+    @property
+    def main_intent(self) -> IntentAPI:
+        return (self.portal.main_intent if self.portal
+                else self.az.intent)
 
     def reply(self, message: str, allow_html: bool = False, render_markdown: bool = True
               ) -> Awaitable[EventID]:
@@ -135,12 +154,15 @@ class CommandEvent:
         message = self._replace_command_prefix(message)
         html = self._render_message(message, allow_html=allow_html,
                                     render_markdown=render_markdown)
+        if self.has_bridge_bot:
+            return self.az.intent.send_notice(self.room_id, message, html=html)
+        else:
+            return self.main_intent.send_notice(self.room_id, message, html=html)
 
-        return self.az.intent.send_notice(self.room_id, message, html=html)
-
-    def mark_read(self) -> Awaitable[None]:
+    async def mark_read(self) -> None:
         """Marks the command as read by the bot."""
-        return self.az.intent.mark_read(self.room_id, self.event_id)
+        if self.has_bridge_bot:
+            await self.az.intent.mark_read(self.room_id, self.event_id)
 
     def _replace_command_prefix(self, message: str) -> str:
         """Returns the string with the proper command prefix entered."""
@@ -184,20 +206,26 @@ class CommandHandler:
         name: The name of this command.
         help_section: Section of the help in which this command will appear.
     """
-    management_only: bool
     name: str
 
+    management_only: bool
+    needs_admin: bool
+    needs_auth: bool
+
     _help_text: str
     _help_args: str
     help_section: HelpSection
 
     def __init__(self, handler: CommandHandlerFunc, management_only: bool, name: str,
-                 help_text: str, help_args: str, help_section: HelpSection, **kwargs) -> None:
+                 help_text: str, help_args: str, help_section: HelpSection,
+                 needs_auth: bool, needs_admin: bool, **kwargs) -> None:
         """
         Args:
             handler: The function handling the execution of this command.
             management_only: Whether the command can exclusively be issued
                 in a management room.
+            needs_auth: Whether the command needs the bridge to be authed already
+            needs_admin: Whether the command needs the issuer to be bridge admin
             name: The name of this command.
             help_text: The text displayed in the help for this command.
             help_args: Help text for the arguments of this command.
@@ -207,6 +235,8 @@ class CommandHandler:
             setattr(self, key, value)
         self._handler = handler
         self.management_only = management_only
+        self.needs_admin = needs_admin
+        self.needs_auth = needs_auth
         self.name = name
         self._help_text = help_text
         self._help_args = help_args
@@ -224,6 +254,10 @@ class CommandHandler:
         if self.management_only and not evt.is_management:
             return (f"`{evt.command}` is a restricted command: "
                     "you may only run it in management rooms.")
+        elif self.needs_admin and not evt.sender.is_admin:
+            return "This command requires administrator privileges."
+        elif self.needs_auth and not await evt.sender.is_logged_in():
+            return "This command requires you to be logged in."
         return None
 
     def has_permission(self, key: HelpCacheKey) -> bool:
@@ -236,7 +270,9 @@ class CommandHandler:
             True if a user with the given state is allowed to issue the
             command.
         """
-        return not self.management_only or key.is_management
+        return ((not self.management_only or key.is_management) and
+                (not self.needs_admin or key.is_admin) and
+                (not self.needs_auth or key.is_logged_in))
 
     async def __call__(self, evt: CommandEvent) -> Any:
         """Executes the command if evt was issued with proper rights.
@@ -267,13 +303,15 @@ def command_handler(_func: Optional[CommandHandlerFunc] = None, *, management_on
                     name: Optional[str] = None, help_text: str = "", help_args: str = "",
                     help_section: HelpSection = None, aliases: Optional[List[str]] = None,
                     _handler_class: Type[CommandHandler] = CommandHandler,
+                    needs_auth: bool = True, needs_admin: bool = False,
                     **kwargs) -> Callable[[CommandHandlerFunc], CommandHandler]:
     """Decorator to create CommandHandlers"""
 
     def decorator(func: CommandHandlerFunc) -> CommandHandler:
         actual_name = name or func.__name__.replace("_", "-")
-        handler = _handler_class(func, management_only, actual_name, help_text, help_args,
-                                 help_section, **kwargs)
+        handler = _handler_class(func, management_only=management_only, name=actual_name,
+                                 help_text=help_text, help_args=help_args, help_section=help_section,
+                                 needs_auth=needs_auth, needs_admin=needs_admin, **kwargs)
         command_handlers[handler.name] = handler
         if aliases:
             for alias in aliases:
@@ -317,9 +355,10 @@ class CommandProcessor:
                      ) -> Awaitable[Any]:
         return handler(evt)
 
-    async def handle(self, room_id: RoomID, event_id: EventID, sender: 'BaseUser',
+    async def handle(self, room_id: RoomID, event_id: EventID, sender: BaseUser,
                      command: str, args: List[str], content: MessageEventContent,
-                     is_management: bool, is_portal: bool) -> None:
+                     portal: Optional[BasePortal], is_management: bool, has_bridge_bot: bool,
+                     ) -> None:
         """Handles the raw commands issued by a user to the Matrix bot.
 
         If the command is not known, it might be a followup command and is
@@ -333,8 +372,9 @@ class CommandProcessor:
             command: The issued command, case insensitive.
             args: Arguments given with the command.
             content: The raw content in the command event.
+            portal: The portal the command was sent to.
             is_management: Whether the room is a management room.
-            is_portal: Whether the room is a portal.
+            has_bridge_bot: Whether or not the bridge bot is in the room.
 
         Returns:
             The result of the error message function or None if no error
@@ -343,8 +383,9 @@ class CommandProcessor:
         if not command_handlers or "unknown-command" not in command_handlers:
             raise ValueError("command_handlers are not properly initialized.")
 
-        evt = self.event_class(self, room_id, event_id, sender, command, args, content,
-                               is_management, is_portal)
+        evt = self.event_class(processor=self, room_id=room_id, event_id=event_id, sender=sender,
+                               command=command, args=args, content=content, portal=portal,
+                               is_management=is_management, has_bridge_bot=has_bridge_bot)
         orig_command = command
         command = command.lower()
         try:
