@@ -14,8 +14,8 @@ from yarl import URL
 from mautrix.types import (EventID, RoomID, UserID, Event, EventType, MessageEvent, MessageType,
                            MessageEventContent, StateEvent, Membership, MemberStateEventContent,
                            PresenceEvent, TypingEvent, ReceiptEvent, TextMessageEventContent,
-                           EncryptedEvent)
-from mautrix.errors import IntentError, MatrixError, MForbidden, DecryptionError
+                           EncryptedEvent, ReceiptType, SingleReceiptEventContent)
+from mautrix.errors import IntentError, MatrixError, MForbidden, DecryptionError, SessionNotFound
 from mautrix.appservice import AppService
 from mautrix.util.logging import TraceLogger
 from mautrix.util.opt_prometheus import Histogram
@@ -60,16 +60,13 @@ class BaseMatrixHandler:
         if self.config["bridge.encryption.allow"]:
             if not EncryptionManager:
                 self.log.error("Encryption enabled in config, but dependencies not installed.")
-            elif not self.config["bridge.login_shared_secret"]:
-                self.log.warning("Encryption enabled in config, but login_shared_secret not set.")
-            else:
-                self.e2ee = EncryptionManager(
-                    bridge=bridge,
-                    user_id_prefix=self.user_id_prefix, user_id_suffix=self.user_id_suffix,
-                    login_shared_secret=self.config["bridge.login_shared_secret"],
-                    homeserver_address=self.config["homeserver.address"],
-                    db_url=self._get_db_url(bridge.config),
-                    key_sharing_config=self.config["bridge.encryption.key_sharing"])
+                return
+            self.e2ee = EncryptionManager(
+                bridge=bridge,
+                user_id_prefix=self.user_id_prefix, user_id_suffix=self.user_id_suffix,
+                homeserver_address=self.config["homeserver.address"],
+                db_url=self._get_db_url(bridge.config),
+                key_sharing_config=self.config["bridge.encryption.key_sharing"])
 
     @staticmethod
     def _get_db_url(config: 'BaseBridgeConfig') -> str:
@@ -126,6 +123,12 @@ class BaseMatrixHandler:
 
     async def init_encryption(self) -> None:
         if self.e2ee:
+            if not await self.e2ee.check_server_support():
+                # Element iOS is broken, so the devs decided to break Synapse as well, which means
+                # there's no reliable way to check if appservice login is supported.
+                # Print a warning in the logs, then try to log in anyway and hope for the best.
+                self.log.warning("Encryption enabled in config, but homeserver does not advertise "
+                                 "appservice login")
             await self.e2ee.start()
 
     @staticmethod
@@ -188,7 +191,8 @@ class BaseMatrixHandler:
 
     async def handle_ephemeral_event(self, evt: Union[ReceiptEvent, PresenceEvent, TypingEvent]
                                      ) -> None:
-        pass
+        if evt.type == EventType.RECEIPT:
+            await self.handle_receipt(evt)
 
     async def send_permission_error(self, room_id: RoomID) -> None:
         await self.az.intent.send_notice(
@@ -284,7 +288,8 @@ class BaseMatrixHandler:
             self.log.trace("Ignoring command %s from %s", event_id, sender.mxid)
             return
 
-        is_management = await self.is_management(room_id)
+        has_two_members, bridge_bot_in_room = await self._is_direct_chat(room_id)
+        is_management = has_two_members and bridge_bot_in_room
 
         if is_command or is_management:
             try:
@@ -294,21 +299,38 @@ class BaseMatrixHandler:
                 # Not enough values to unpack, i.e. no arguments
                 command = text
                 args = []
-            await self.commands.handle(room_id, event_id, sender, command, args, message,
-                                       is_management, is_portal=portal is not None)
+            await self.commands.handle(room_id, event_id, sender, command, args, message, portal,
+                                       is_management, bridge_bot_in_room)
         else:
             self.log.trace("Ignoring event %s from %s: not a command and not a portal room",
                            event_id, sender.mxid)
 
-    async def is_management(self, room_id: RoomID) -> bool:
+    async def _is_direct_chat(self, room_id: RoomID) -> Tuple[bool, bool]:
         try:
             members = await self.az.intent.get_room_members(room_id)
-            return len(members) == 2 and self.az.bot_mxid in members
+            return len(members) == 2, self.az.bot_mxid in members
         except MatrixError:
-            return False
+            return False, False
+
+    async def handle_receipt(self, evt: ReceiptEvent) -> None:
+        for event_id, receipts in evt.content.items():
+            for user_id, data in receipts[ReceiptType.READ].items():
+                user = await self.bridge.get_user(user_id, create=False)
+                if not user or not await user.is_logged_in():
+                    continue
+
+                portal = await self.bridge.get_portal(evt.room_id)
+                if not portal:
+                    continue
+
+                await self.handle_read_receipt(user, portal, event_id, data)
+
+    async def handle_read_receipt(self, user: 'BaseUser', portal: 'BasePortal', event_id: EventID,
+                                  data: SingleReceiptEventContent) -> None:
+        pass
 
     def filter_matrix_event(self, evt: Event) -> bool:
-        if not isinstance(evt, (MessageEvent, StateEvent)):
+        if not isinstance(evt, (MessageEvent, StateEvent, ReceiptEvent)):
             return False
         return evt.sender == self.az.bot_mxid
 
@@ -324,12 +346,13 @@ class BaseMatrixHandler:
     async def send_encryption_error_notice(self, evt: EncryptedEvent,
                                            error: DecryptionError) -> None:
         await self.az.intent.send_notice(evt.room_id,
-                                         f"\u26a0 Your message was not bridged: {error}. Try "
-                                         f"restarting your client if this error keeps happening.")
+                                         f"\u26a0 Your message was not bridged: {error}")
 
     async def handle_encrypted(self, evt: EncryptedEvent) -> None:
         try:
-            decrypted = await self.e2ee.decrypt(evt)
+            decrypted = await self.e2ee.decrypt(evt, wait_session_timeout=5)
+        except SessionNotFound as e:
+            await self._handle_encrypted_wait(evt, e, wait=10)
         except DecryptionError as e:
             self.log.warning("Failed to decrypt event %s: %s", evt.event_id, e)
             self.log.trace("%s decryption traceback:", evt.event_id, exc_info=True)
@@ -337,10 +360,35 @@ class BaseMatrixHandler:
         else:
             await self.int_handle_event(decrypted)
 
+    async def _handle_encrypted_wait(self, evt: EncryptedEvent, err: SessionNotFound, wait: int
+                                     ) -> None:
+        self.log.warning(f"Didn't find session {err.session_id}, waiting even longer")
+        msg = ("\u26a0 Your message was not bridged: the bridge hasn't received the decryption "
+               f"keys. The bridge will retry for {wait} seconds. If this error keeps happening, "
+               "try restarting your client.")
+        event_id = await self.az.intent.send_notice(evt.room_id, msg)
+        got_keys = await self.e2ee.crypto.wait_for_session(evt.room_id, err.sender_key,
+                                                           err.session_id, timeout=wait)
+        if got_keys:
+            try:
+                decrypted = await self.e2ee.decrypt(evt, wait_session_timeout=0)
+            except DecryptionError as e:
+                msg = f"\u26a0 Your message was not bridged: {e}"
+            else:
+                await self.az.intent.redact(evt.room_id, event_id)
+                await self.int_handle_event(decrypted)
+                return
+        else:
+            msg = ("\u26a0 Your message was not bridged: the bridge hasn't received the decryption "
+                   "keys. If this error keeps happening, try restarting your client.")
+        content = TextMessageEventContent(msgtype=MessageType.NOTICE, body=msg)
+        content.set_edit(event_id)
+        await self.az.intent.send_message(evt.room_id, content)
+
     async def handle_encryption(self, evt: StateEvent) -> None:
         await self.az.state_store.set_encryption_info(evt.room_id, evt.content)
         portal = await self.bridge.get_portal(evt.room_id)
-        if portal.encrypted:
+        if portal:
             portal.encrypted = True
             await portal.save()
 
@@ -396,6 +444,9 @@ class BaseMatrixHandler:
         else:
             if evt.type.is_state and isinstance(evt, StateEvent):
                 await self.handle_state_event(evt)
+            elif evt.type.is_ephemeral and isinstance(evt, (PresenceEvent, TypingEvent,
+                                                            ReceiptEvent)):
+                await self.handle_ephemeral_event(evt)
             else:
                 await self.handle_event(evt)
 
